<?php

namespace Drupal\document_ocr\Plugin\document_ocr\processor;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\TempStore\PrivateTempStoreFactory;
use Drupal\Core\File\FileSystemInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\document_ocr\Plugin\ProcessorBase;
use Drupal\document_ocr\Services\OpenAi as OpenAiService;

/**
 * OpenAiAudio processor plugin.
 *
 * @DocumentOcrProcessor(
 *   id = "openai_audio",
 *   name = @Translation("OpenAI Audio"),
 *   extensions = "mp3 mp4 mpeg mpga m4a wav webm",
 *   require_credentials = true,
 *   require_template = false
 * )
 */
class OpenAiAudio extends ProcessorBase {

  /**
   * OpenAI serivce.
   *
   * @var \Drupal\document_ocr\Services\OpenAi as OpenAiService
   */
  protected $openAi;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, PrivateTempStoreFactory $temp_store_factory, FileSystemInterface $file_system, OpenAiService $openai) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $temp_store_factory, $file_system);
    $this->openAi = $openai;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static($configuration, $plugin_id, $plugin_definition,
      $container->get('tempstore.private'),
      $container->get('file_system'),
      $container->get('document_ocr.openai')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function requirementsAreMet() {
    return class_exists('\OpenAI\Client');
  }

  /**
   * {@inheritdoc}
   */
  public function configurationForm(array $form, FormStateInterface $form_state) {
    $configuration = $this->getConfiguration();
    $form['model'] = [
      '#title' => $this->t('Model'),
      '#type' => 'select',
      '#empty_option' => $this->t('- Select OpenAI model -'),
      '#description' => $this->t('If not selected default model will be used.'),
      '#validated' => TRUE,
      '#options' => [
        'whisper-1' => 'whisper-1',
      ],
      '#default_value' => !empty($configuration['model']) ? $configuration['model'] : 'whisper-1',
    ];
    $form['task'] = [
      '#type' => 'select',
      '#title' => $this->t('Task'),
      '#options' => [
        'transcribe' => $this->t('Transcribe'),
        'translate' => $this->t('Translate'),
      ],
      '#default_value' => 'transcribe',
      '#description' => $this->t('The task to use to process the audio file. "Transcribe": transcribes the audio to the same language as the audio. "Translate": translates and transcribes the audio into English. See the <a href="@link" target="_blank">speech to text guide</a> for further details.', ['@link' => 'https://platform.openai.com/docs/guides/speech-to-text']),
      '#default_value' => !empty($configuration['task']) ? $configuration['task'] : 'transcribe',
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function configurationValues(array &$form, FormStateInterface $form_state) {
    return [
      'model' => $form_state->getValue('model'),
      'task' => $form_state->getValue('task'),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getMappingData() {
    try {
      $configuration = $this->getConfiguration();
      $credentials = json_decode(file_get_contents(
        $this->fileSystem->realpath($this->getCredentials())
      ), TRUE);
      $client = $this->openAi->setCredentials($credentials);
      // This integration does not require template file, but we still need to pass properties to the mapping tool.
      $this->setOption('extracted_text', $this->t('Extracted Text'));
      if ($file = $this->getFile()) {
        $response = $client->audio($this->fileSystem->realpath($file->getFileUri()), $configuration);
        $document = $response->toArray();
        $this->setOptionValue('extracted_text', $document['text']);
        $this->setDocument($document);
      }
      else {
        $this->setDocument(NULL);
      }
      return $this;
    }
    catch (\Exception $ex) {
      \Drupal::logger('document_ocr')->error($ex->getMessage());
      return FALSE;
    }
  }

}
