<?php

namespace Drupal\document_ocr\Plugin\document_ocr\transformer;

use Drupal\Core\Form\FormStateInterface;
Use Drupal\taxonomy\Entity\Vocabulary;
use Drupal\taxonomy\Entity\Term;

/**
 * Open AI integration transformer plugin.
 *
 * @DocumentOcrTransformer(
 *   id = "openai_keywords",
 *   name = @Translation("OpenAI Keywords (Experimental)"),
 *   description = @Translation("OpenAI integration transformer (extract keywords)."),
 *   require_credentials = true
 * )
 */
class OpenAiKeywords extends OpenAi {

  /**
   * {@inheritdoc}
   */
  public function configurationForm(array $form, FormStateInterface $form_state) {
    $configuration = $this->getConfiguration();
    $array = range(1, 25);
    $form['keywords_limit'] = [
      '#type' => 'select',
      '#title' => $this->t('Keywords Limit'),
      '#description' => $this->t('Number of Keywords to generate from text'),
      '#default_value' => !empty($configuration['keywords_limit']) ? $configuration['keywords_limit'] : 5,
      '#options' => array_combine($array, $array),
      '#required' => TRUE,
    ];
    $options = [];
    $vocabularies = Vocabulary::loadMultiple();
    foreach($vocabularies as $vocabulary) {
      $options[$vocabulary->id()] = $vocabulary->label();
    }
    $form['vocabulary'] = [
      '#type' => 'select',
      '#title' => $this->t('Vocabulary'),
      '#description' => $this->t('Select vocabulary to store keywords. Make sure you specifiy the same vocabulary as the field you will be using in the mapping tool.'),
      '#default_value' => !empty($configuration['vocabulary']) ? $configuration['vocabulary'] : NULL,
      '#options' => $options,
      '#required' => TRUE,
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function configurationValues(array &$form, FormStateInterface $form_state) {
    return [
      'keywords_limit' => $form_state->getValue('keywords_limit'),
      'vocabulary' => $form_state->getValue('vocabulary'),
      'model' => 'text-davinci-003',
      'temperature' => 0.5,
      'max_tokens' => 60,
      'top_p' => 1,
      'frequency_penalty' => 0.8,
      'presence_penalty' => 0,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function transform($value, $configuration) {
    try {
      $this->openAi->setCredentials($this->getCredentials());
      $configuration['prompt'] = "Extract " . (int) $configuration['keywords_limit'] . " keywords from this text separated by comma:\n\n" . $value;
      // The following options should be removed (not part of the OpenAI).
      $vocabulary = $configuration['vocabulary'];
      unset($configuration['keywords_limit'], $configuration['vocabulary']);
      if ($response = $this->openAi->request($configuration)) {
        if (!empty($response->choices[0])) {
          $choice = $response->choices[0]->toArray();
          $target_ids = [];
          if ($e = explode(',', $choice['text'])) {
            foreach ($e as $keyword) {
              if ($term = $this->term($keyword, $vocabulary)) {
                $target_ids[]['target_id'] = $term->id();
              }
            }
          }

          return $target_ids;
        }
      }
    }
    catch (\Exception $ex) {
      return FALSE;
    }
  }

  /**
   * Re-use or create taxonomy term.
   */
  protected function term($keyword, $vocabulary) {
    if (!empty($keyword)) {
      $terms = \Drupal::entityTypeManager()->getStorage('taxonomy_term')->loadByProperties(['name' => $keyword, 'vid' => $vocabulary]); 
      if ($term = reset($terms)) {
        return $term;
      } 
      $term = Term::create(['name' => $keyword, 'vid' => $vocabulary]);
      $term->save();
      return $term;
    }
  }

}
