<?php

namespace Drupal\document_ocr\Services;

use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Cache\CacheBackendInterface;
use Google\ApiCore\ApiException;
use Google\Cloud\Translate\V3\TranslationServiceClient;

/**
 * GoogleTranslate service.
 */
class GoogleTranslate {

  /**
   * The file system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * The Google Translator service.
   *
   * @var \Google\Cloud\Translate\V3\TranslationServiceClient
   */
  protected $client;

  /**
   * The Google project name.
   *
   * @var string
   */
  protected $project_id;

  /**
   * The translator service location.
   *
   * @var string
   */
  protected $location = 'global';

  /**
   * The Google Service credentials.
   *
   * @var string
   */
  protected $credentials;

  /**
   * The cache.default cache backend.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected $cacheBackend;

  /**
   * Cache expiration in seconds.
   */
  protected $interval = 3600;

  /**
   * {@inheritdoc}
   */
  public function __construct(FileSystemInterface $file_system, CacheBackendInterface $cache) {
    $this->fileSystem = $file_system;
    $this->cacheBackend = $cache;
  }

  /**
   * Set JSON credentials file.
   */
  public function setCredentials($credentials) {
    // Create a client.
    $this->credentials = $credentials;
    $this->setProjectId($this->credentials['project_id']);
    return $this;
  }

  /**
   * Initialize servie client.
   */
  public function serviceClient() {
    $this->client = new TranslationServiceClient([
      'credentials' => $this->credentials,
    ]);
    return $this;
  }

  /**
   * Get a project ID.
   */
  public function getProjectId() {
    return $this->project_id;
  }

  /**
   * Set a project ID.
   */
  public function setProjectId($id) {
    $this->project_id = $id;
    return $this;
  }

  /**
   * Get supported languages.
   */
  public function getSupportedLanguages() {
    $languages = [];
    $cid = 'google_translate:languages:' . $this->getProjectId() . ':' . $this->location;
    if ($cache = $this->cacheBackend->get($cid)) {
      $languages = $cache->data;
    }
    else {
      try {
        $standad_languages = \Drupal::languageManager()->getStandardLanguageList();
        $standad_languages['tt'] = ['Tatar', 'Татарча'];
        $standad_languages['uz'] = ['Uzbek', 'Oʻzbekcha'];
        $this->serviceClient();
        $formatted_parent = TranslationServiceClient::locationName($this->getProjectId(), $this->location);
        $response = $this->client->getSupportedLanguages($formatted_parent);
        foreach ($response->getLanguages() as $language) {
          $langcode = $language->getLanguageCode();
          $languages[$langcode] = !empty($standad_languages[$langcode]) ? $this->getDisplayName($standad_languages[$langcode]) : $langcode;
        }
      }
      catch (ApiException $ex) {
        \Drupal::logger('document_ocr')->error($ex->getMessage());
      }
      $this->cacheBackend->set($cid, $languages, time() + $this->interval, [
        'google_translate:languages',
        'google_translate:languages:' . $this->location
      ]);
    }
    return $languages;
  }

  /**
   * Translate text.
   */
  public function translate($text, $language) {
    try {
      $this->serviceClient();
      $formatted_parent = TranslationServiceClient::locationName($this->getProjectId(), $this->location);
      $response = $this->client->translateText([$text], $language, $formatted_parent);
      return $response;
    }
    catch (ApiException $ex) {
      \Drupal::logger('document_ocr')->error($ex->getMessage());
    }
  }

  /**
   * Get language display name.
   */
  protected function getDisplayName($labels) {
    $name = '';
    $variations = [];
    foreach ($labels as $index => $label) {
      if ($index == 0) {
        $name = $label;
      }
      else {
        $variations[] = $label;
      }
    }
    return $name . ' (' . join(', ', $variations) . ')';
  }

}
